@@ -1,20 +1,29 @@
 from __future__ import print_function
-import os,re,subprocess,tempfile,random,sys
+import os,re,subprocess,tempfile,random,sys,glob
 from pkg_resources import parse_version
 from io import StringIO
 import logging as pyLog
+import six
+
 try:
     from string import strip
 except ImportError:
     strip = str.strip
 
+try:
+    import xarray
+    loadedXarray = True
+except:
+    print("Could not load xarray")
+    loadedXarray = False
+
 # Copyright (C) 2011-2012 Ralf Mueller, ralf.mueller@zmaw.de
 # See COPYING file for copying and redistribution conditions.
-# 
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; version 2 of the License.
-# 
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -22,7 +31,7 @@ except ImportError:
 
 CDF_MOD_SCIPY   = "scipy"
 CDF_MOD_NETCDF4 = "netcdf4"
-CDO_PY_VERSION  = "1.3.0"
+CDO_PY_VERSION  = "1.3.5"
 
 def auto_doc(tool, cdo_self):
     """Generate the __doc__ string of the decorated function by calling the cdo help command"""
@@ -31,10 +40,12 @@ def auto_doc(tool, cdo_self):
         return func
     return desc
 
-def getCdoVersion(path2cdo):
+def getCdoVersion(path2cdo,verbose=False):
     proc = subprocess.Popen([path2cdo,'-V'],stderr = subprocess.PIPE,stdout = subprocess.PIPE)
     ret  = proc.communicate()
     cdo_help   = ret[1].decode("utf-8")
+    if verbose:
+        return cdo_help
     match = re.search("Climate Data Operators version (\d.*) .*",cdo_help)
     return match.group(1)
 
@@ -42,7 +53,7 @@ def setupLogging(logFile):
     logger = pyLog.getLogger(__name__)
     logger.setLevel(pyLog.INFO)
 
-    if isinstance(logFile,str):
+    if isinstance(logFile, six.string_types):
         handler = pyLog.FileHandler(logFile)
     else:
         handler = pyLog.StreamHandler(stream=logFile)
@@ -78,30 +89,6 @@ class Cdo(object):
                logging=False,
                logFile=StringIO()):
 
-    # Since cdo-1.5.4 undocumented operators are given with the -h option. For
-    # earlier version, they have to be provided manually
-    self.undocumentedOperators = ['anomaly','beta','boxavg','change_e5lsm','change_e5mask',
-        'change_e5slm','chisquare','chvar','cloudlayer','cmd','com','command','complextorect',
-        'covar0','covar0r','daycount','daylogs','del29feb','delday','delete','deltap','deltap_fl',
-        'delvar','diffv','divcoslat','dumplogo','dumplogs','duplicate','eca_r1mm','enlargegrid',
-        'ensrkhistspace','ensrkhisttime','eof3d','eof3dspatial','eof3dtime','export_e5ml',
-        'export_e5res','fc2gp','fc2sp','fillmiss','fisher','fldcovar','fldrms','fourier','fpressure',
-        'gather','gengrid','geopotheight','ggstat','ggstats','globavg','gp2fc','gradsdes',
-        'gridverify','harmonic','hourcount','hpressure','ifs2icon','import_e5ml','import_e5res',
-        'import_obs','imtocomplex','infos','infov','interpolate','intgrid','intgridbil',
-        'intgridtraj','intpoint','isosurface','lmavg','lmean','lmmean','lmstd','log','lsmean',
-        'meandiff2test','mergegrid','mod','moncount','monlogs','mrotuv','mrotuvb','mulcoslat','ncode',
-        'ncopy','nmltest','normal','nvar','outputbounds','outputboundscpt','outputcenter',
-        'outputcenter2','outputcentercpt','outputkey','outputtri','outputvector','outputvrml',
-        'pardup','parmul','pinfo','pinfov','pressure_fl','pressure_hl','read_e5ml','remapcon1',
-        'remapdis1','retocomplex','scalllogo','scatter','seascount','select','selgridname',
-        'seloperator','selvar','selzaxisname','setrcaname','setvar','showvar','sinfov','smemlogo',
-        'snamelogo','sort','sortcode','sortlevel','sortname','sorttaxis','sorttimestamp','sortvar',
-        'sp2fc','specinfo','spectrum','sperclogo','splitvar','stimelogo','studentt','template1',
-        'template2','test','test2','testdata','thinout','timcount','timcovar','tinfo','transxy','trms',
-        'tstepcount','vardes','vardup','varmul','varquot2test','varrms','vertwind','write_e5ml',
-        'writegrid','writerandom','yearcount']
-
     if 'CDO' in os.environ:
       self.CDO = os.environ['CDO']
     else:
@@ -115,7 +102,7 @@ class Cdo(object):
     self.cdfMod                 = cdfMod.lower()
     self.env                    = env
     self.debug                  = True if 'DEBUG' in os.environ else debug
-    self.outputOperators        = ['cdiread','cmor','codetab','conv_cmor_table','diff','diffc','diffn','diffp','diffv','dump_cmor_table','dumpmap','filedes','ggstat','ggstats','gmtcells','gmtxyz','gradsdes','griddes','griddes2','gridverify','info','infoc','infon','infop','infos','infov','map','ncode','ncode','ndate','ngridpoints','ngrids','nlevel','nmon','npar','ntime','nvar','nyear','output','outputarr','outputbounds','outputboundscpt','outputcenter','outputcenter2','outputcentercpt','outputext','outputf','outputfld','outputint','outputkey','outputsrv','outputtab','outputtri','outputts','outputvector','outputvrml','outputxyz','pardes','partab','partab2','seinfo','seinfoc','seinfon','seinfop','showcode','showdate','showformat','showlevel','showltype','showmon','showname','showparam','showstdname','showtime','showtimestamp','showunit','showvar','showyear','sinfo','sinfoc','sinfon','sinfop','sinfov','spartab','specinfo','tinfo','vardes','vct','vct2','verifygrid','vlist','zaxisdes']
+    self.noOutputOperators      = self.getNoOutputOperators()
     self.libs                   = self.getSupportedLibs()
 
     self.logging                = logging
@@ -128,23 +115,18 @@ class Cdo(object):
     res.extend(self.operators)
     return res
 
-  def isString(self,myString):
-      if (2 == sys.version_info[0]):
-          return isinstance(myString,basestring)
-      else:
-          return isinstance(myString,str)
-
-  def call(self,cmd):
+  def call(self,cmd,envOfCall={}):
     if self.logging and '-h' != cmd[1]:
       self.logger.info(u' '.join(cmd))
 
-    for k,v in self.env.items():
-      os.environ[k] = v
+    env = dict(self.env)
+    env.update(envOfCall)
 
     proc = subprocess.Popen(' '.join(cmd),
                             shell  = True,
                             stderr = subprocess.PIPE,
-                            stdout = subprocess.PIPE)
+                            stdout = subprocess.PIPE,
+                            env    = env)
 
     retvals = proc.communicate()
     stdout  = retvals[0].decode("utf-8")
@@ -152,8 +134,8 @@ class Cdo(object):
 
     if self.debug:
       print('# DEBUG - start =============================================================')
-      if {} != self.env:
-        for k,v in list(self.env.items()):
+      if {} != env:
+        for k,v in list(env.items()):
           print("ENV: " + k + " = " + v)
       print('CALL  :' + ' '.join(cmd))
       print('STDOUT:')
@@ -188,7 +170,9 @@ class Cdo(object):
     @auto_doc(method_name, self)
     def get(self, *args,**kwargs):
       operator          = [method_name]
-      operatorPrintsOut = method_name in self.outputOperators
+      operatorPrintsOut = method_name in self.noOutputOperators
+
+      self.envByCall = {}
 
       if args.__len__() != 0:
         for arg in args:
@@ -197,29 +181,59 @@ class Cdo(object):
       #build the cdo command
       #0. the cdo command
       cmd = [self.CDO]
+
       #1. OVERWRITE EXISTING FILES
       cmd.append('-O')
+
       #2. options
       if 'options' in kwargs:
           cmd += kwargs['options'].split()
+
       #3. operator(s)
       cmd.append(','.join(operator))
+
       #4. input files or operators
       if 'input' in kwargs:
-        if self.isString(kwargs["input"]):
+        if isinstance(kwargs["input"], six.string_types):
+            cmd.append(kwargs["input"])
+        elif type(kwargs["input"]) == list:
+            cmd.append(' '.join(kwargs["input"]))
+        elif (True == loadedXarray and type(kwargs["input"]) == xarray.core.dataset.Dataset):
+
+            # create a temp nc file from input data
+            tempfile = MyTempfile()
+            _tpath = tempfile.path()
+            kwargs["input"].to_netcdf(_tpath)
+            kwargs["input"] = _tpath
+            print(kwargs['input'])
             cmd.append(kwargs["input"])
         else:
             #we assume it's either a list, a tuple or any iterable.
-            cmd += kwargs["input"]
+            cmd.append(kwargs["input"])
 
+      #5. handle rewrite of existing output files
       if not kwargs.__contains__("force"):
         kwargs["force"] = self.forceOutput
 
+      #6. handle environment setup per call
+      envOfCall = {}
+      if kwargs.__contains__("env"):
+        for k,v in kwargs["env"].items():
+          envOfCall[k] = v
+
       if operatorPrintsOut:
-        retvals = self.call(cmd)
+        retvals = self.call(cmd,envOfCall)
         if ( not self.hasError(method_name,cmd,retvals) ):
           r = list(map(strip,retvals["stdout"].split(os.linesep)))
-          return r[:len(r)-1]
+          if "autoSplit" in kwargs:
+            splitString = kwargs["autoSplit"]
+            _output = [x.split(splitString) for x in r[:len(r)-1]]
+            if (1 == len(_output)):
+                return _output[0]
+            else:
+                return _output
+          else:
+           return r[:len(r)-1]
         else:
           if self.returnNoneOnError:
             return None
@@ -232,11 +246,8 @@ class Cdo(object):
             kwargs["output"] = self.tempfile.path()
 
           cmd.append(kwargs["output"])
-          if kwargs.__contains__("env"):
-            for k,v in kwargs["env"].items():
-              os.environ[k] = v
 
-          retvals = self.call(cmd)
+          retvals = self.call(cmd,envOfCall)
           if self.hasError(method_name,cmd,retvals):
             if self.returnNoneOnError:
               return None
@@ -251,10 +262,22 @@ class Cdo(object):
 
       if not None == kwargs.get("returnArray"):
         return self.readArray(kwargs["output"],kwargs["returnArray"])
+
       elif not None == kwargs.get("returnMaArray"):
         return self.readMaArray(kwargs["output"],kwargs["returnMaArray"])
+
       elif self.returnCdf or kwargs["returnCdf"]:
         return self.readCdf(kwargs["output"])
+
+      elif loadedXarray and not None == kwargs.get("returnXArray"):
+        return self.readXArray(kwargs["output"],kwargs.get("returnXArray"))
+
+      elif loadedXarray and not None == kwargs.get("returnXDataset"):
+        return self.readXDataset(kwargs["output"])
+
+      elif ('split' == method_name[0:5]):
+        return glob.glob(kwargs["output"]+'*')
+
       else:
         return kwargs["output"]
 
@@ -276,7 +299,6 @@ class Cdo(object):
           raise AttributeError("Unknown method '" + method_name +"'!")
 
   def getOperators(self):
-    import os
     if (parse_version(getCdoVersion(self.CDO)) > parse_version('1.7.0')):
         proc = subprocess.Popen([self.CDO,'--operators'],stderr = subprocess.PIPE,stdout = subprocess.PIPE)
         ret  = proc.communicate()
@@ -293,7 +315,29 @@ class Cdo(object):
         s    = ' '.join(ops[:endI]).strip()
         s    = re.sub("\s+" , " ", s)
 
-        return list(set(s.split(" ") + self.undocumentedOperators))
+        return list(set(s.split(" ")))
+
+  def getNoOutputOperators(self):
+    if ( \
+            parse_version(getCdoVersion(self.CDO)) > parse_version('1.8.0') and \
+            parse_version(getCdoVersion(self.CDO)) < parse_version('1.9.0') ):
+      proc = subprocess.Popen([self.CDO,'--operators_no_output'],stderr = subprocess.PIPE,stdout = subprocess.PIPE)
+      ret  = proc.communicate()
+      return list(map(lambda x : x.split(' ')[0], ret[0].decode("utf-8")[0:-1].split(os.linesep)))
+    else:
+      return ['cdiread','cmor','codetab','conv_cmor_table','diff','diffc','diffn','diffp'
+              ,'diffv','dump_cmor_table','dumpmap','filedes','ggstat','ggstats','gmtcells'
+              ,'gmtxyz','gradsdes','griddes','griddes2','gridverify','info','infoc','infon'
+              ,'infop','infos','infov','map','ncode','ncode','ndate','ngridpoints','ngrids'
+              ,'nlevel','nmon','npar','ntime','nvar','nyear','output','outputarr','outputbounds'
+              ,'outputboundscpt','outputcenter','outputcenter2','outputcentercpt','outputext'
+              ,'outputf','outputfld','outputint','outputkey','outputsrv','outputtab','outputtri'
+              ,'outputts','outputvector','outputvrml','outputxyz','pardes','partab','partab2'
+              ,'seinfo','seinfoc','seinfon','seinfop','showcode','showdate','showformat','showlevel'
+              ,'showltype','showmon','showname','showparam','showstdname','showtime','showtimestamp'
+              ,'showunit','showvar','showyear','sinfo','sinfoc','sinfon','sinfop','sinfov'
+              ,'spartab','specinfo','tinfo','vardes','vct','vct2','verifygrid','vlist','zaxisdes']
+
 
   def loadCdf(self):
     if self.cdfMod == CDF_MOD_SCIPY:
@@ -354,7 +398,7 @@ class Cdo(object):
       self.setReturnArray(False)
 
   def collectLogs(self):
-      if isinstance(self.logFile,str):
+      if isinstance(self.logFile, six.string_types):
           content = []
           with open(self.logFile,'r') as f:
               content.append(f.read())
@@ -407,9 +451,9 @@ class Cdo(object):
   #==================================================================
   # Addional operators:
   #------------------------------------------------------------------
-  def version(self):
+  def version(self,verbose=False):
     # return CDO's version
-    return getCdoVersion(self.CDO)
+    return getCdoVersion(self.CDO,verbose)
 
   def boundaryLevels(self,**kwargs):
     ilevels         = list(map(float,self.showlevel(input = kwargs['input'])[0].split()))
@@ -484,8 +528,22 @@ class Cdo(object):
 
     return retval
 
+  def readXArray(self,ifile,varname):
+    dataSet = xarray.open_dataset(ifile)
+    return dataSet[varname]
+
+  def readXDataset(self,ifile):
+    return xarray.open_dataset(ifile)
+
   def __version__(self):
     return CDO_PY_VERSION
+
+  def __print__(self,context=''):
+    if '' != context:
+      print('CDO:CONTEXT '+context)
+    print("CDO:ID  = "+str(id(self)))
+    print("CDO:ENV = "+str(self.env))
+
 # Helper module for easy temp file handling
 class MyTempfile(object):
 
@@ -511,5 +569,5 @@ class MyTempfile(object):
 
       return t.name
     else:
-      N =10000000 
+      N =10000000
       t = "_"+random.randint(N).__str__()
