@@ -51,7 +51,7 @@ class CdoTest(unittest.TestCase):
 
     def testVersions(self):
         cdo = Cdo()
-        self.assertEqual('1.3.3',cdo.__version__())
+        self.assertEqual('1.3.5',cdo.__version__())
         self.assertTrue(parse_version('1.7.0') <= parse_version(cdo.version()))
 
     def testCDO(self):
@@ -103,7 +103,7 @@ class CdoTest(unittest.TestCase):
         s   = cdo.sinfov(input="-topo",options="-f nc")
         s   = cdo.sinfov(input="-remapnn,r36x18 -topo",options="-f nc")
         f   = 'test_ofile.nc'
-        cdo.expr("'z=log(abs(topo+1))*9.81'",input="-topo", output=f, options="-f nc")
+        cdo.expr("'z=log(abs(topo)+1)*9.81'",input="-topo", output=f, options="-f nc")
         s   = cdo.infov(input=f)
         cdo.stdatm("0",output=f,options="-f nc")
         rm([f,])
@@ -167,12 +167,10 @@ class CdoTest(unittest.TestCase):
     def test_diff(self):
         cdo = Cdo(cdfMod=CDF_MOD)
         cdo.debug = DEBUG
-        diffv = cdo.diffn(input = "-random,r1x1 -random,r1x1")
+        diffv = cdo.diffn(input = "-random,global_0.1 -random,global_0.1")
         print(diffv)
         self.assertEqual(diffv[1].split(' ')[-1],"random")
-        self.assertEqual(diffv[1].split(' ')[-3],"0.53060")
-        diff  = cdo.diff(input = "-random,r1x1 -random,r1x1")
-        self.assertEqual(diff[1].split(' ')[-3],"0.53060")
+        self.assertEqual(diffv[1].split(' ')[-3],"1.0000")
 
     def test_returnCdf(self):
         cdo = Cdo(cdfMod=CDF_MOD)
@@ -658,6 +656,19 @@ class CdoTest(unittest.TestCase):
       self.assertTrue(opCount > 50)
       self.assertTrue(opCount < 200)
 
+    def test_cdiMeta(self):
+      cdo = Cdo()
+      ofile = cdo.stdatm("0", options = "-f nc", returnCdf = True)
+      print(ofile)
+      ofile = cdo.stdatm("0", options = "-f nc4", returnCdf = True)
+      print(ofile)
+      ofile = cdo.stdatm("0", options = "-f nc", returnXArray = 'T')
+      print(ofile)
+      print(ofile.attrs)
+      ofile = cdo.stdatm("0", options = "-f nc", returnXDataset=True)
+      print(ofile)
+      print(ofile.attrs)
+
     if MAINTAINERMODE:
         def test_icon_coords(self):
           cdo = Cdo(cdfMod=CDF_MOD)
